package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import org.apache.bcel.Const;

/* loaded from: org_apache_bcel_classfile.jar:org/apache/bcel/classfile/ConstantPool.class */
public class ConstantPool implements Cloneable, Node, Iterable<Constant> {
    private Constant[] constantPool;

    private static String escape(String str) {
        int len = str.length();
        StringBuilder buf = new StringBuilder(len + 5);
        char[] ch = str.toCharArray();
        for (int i = 0; i < len; i++) {
            switch (ch[i]) {
                case '\b':
                    buf.append("\\b");
                    break;
                case '\t':
                    buf.append("\\t");
                    break;
                case '\n':
                    buf.append("\\n");
                    break;
                case '\r':
                    buf.append("\\r");
                    break;
                case '\"':
                    buf.append("\\\"");
                    break;
                default:
                    buf.append(ch[i]);
                    break;
            }
        }
        return buf.toString();
    }

    public ConstantPool(Constant[] constantPool) {
        this.constantPool = constantPool;
    }

    public ConstantPool(DataInput input) throws IOException {
        int constantPoolCount = input.readUnsignedShort();
        this.constantPool = new Constant[constantPoolCount];
        int i = 1;
        while (i < constantPoolCount) {
            this.constantPool[i] = Constant.readConstant(input);
            byte tag = this.constantPool[i].getTag();
            if (tag == 6 || tag == 5) {
                i++;
            }
            i++;
        }
    }

    @Override // org.apache.bcel.classfile.Node
    public void accept(Visitor v) {
        v.visitConstantPool(this);
    }

    public String constantToString(Constant c) throws IllegalArgumentException {
        String str;
        byte tag = c.getTag();
        switch (tag) {
            case JavaClass.HEAP /* 1 */:
                str = ((ConstantUtf8) c).getBytes();
                break;
            case JavaClass.FILE /* 2 */:
            case 13:
            case 14:
            default:
                throw new IllegalArgumentException("Unknown constant type " + ((int) tag));
            case JavaClass.ZIP /* 3 */:
                str = String.valueOf(((ConstantInteger) c).getBytes());
                break;
            case 4:
                str = String.valueOf(((ConstantFloat) c).getBytes());
                break;
            case 5:
                str = String.valueOf(((ConstantLong) c).getBytes());
                break;
            case 6:
                str = String.valueOf(((ConstantDouble) c).getBytes());
                break;
            case 7:
                int i = ((ConstantClass) c).getNameIndex();
                str = Utility.compactClassName(getConstantUtf8(i).getBytes(), false);
                break;
            case 8:
                int i2 = ((ConstantString) c).getStringIndex();
                str = "\"" + escape(getConstantUtf8(i2).getBytes()) + "\"";
                break;
            case 9:
            case 10:
            case 11:
                str = constantToString(((ConstantCP) c).getClassIndex(), (byte) 7) + "." + constantToString(((ConstantCP) c).getNameAndTypeIndex(), (byte) 12);
                break;
            case 12:
                str = constantToString(((ConstantNameAndType) c).getNameIndex(), (byte) 1) + " " + constantToString(((ConstantNameAndType) c).getSignatureIndex(), (byte) 1);
                break;
            case 15:
                ConstantMethodHandle cmh = (ConstantMethodHandle) c;
                str = Const.getMethodHandleName(cmh.getReferenceKind()) + " " + constantToString(cmh.getReferenceIndex(), getConstant(cmh.getReferenceIndex()).getTag());
                break;
            case 16:
                ConstantMethodType cmt = (ConstantMethodType) c;
                str = constantToString(cmt.getDescriptorIndex(), (byte) 1);
                break;
            case 17:
                ConstantDynamic cd = (ConstantDynamic) c;
                str = cd.getBootstrapMethodAttrIndex() + ":" + constantToString(cd.getNameAndTypeIndex(), (byte) 12);
                break;
            case 18:
                ConstantInvokeDynamic cid = (ConstantInvokeDynamic) c;
                str = cid.getBootstrapMethodAttrIndex() + ":" + constantToString(cid.getNameAndTypeIndex(), (byte) 12);
                break;
            case 19:
                int i3 = ((ConstantModule) c).getNameIndex();
                str = Utility.compactClassName(getConstantUtf8(i3).getBytes(), false);
                break;
            case 20:
                int i4 = ((ConstantPackage) c).getNameIndex();
                str = Utility.compactClassName(getConstantUtf8(i4).getBytes(), false);
                break;
        }
        return str;
    }

    public String constantToString(int index, byte tag) {
        return constantToString(getConstant(index, tag));
    }

    public ConstantPool copy() {
        ConstantPool c = null;
        try {
            c = (ConstantPool) clone();
            c.constantPool = new Constant[this.constantPool.length];
            for (int i = 1; i < this.constantPool.length; i++) {
                if (this.constantPool[i] != null) {
                    c.constantPool[i] = this.constantPool[i].copy();
                }
            }
        } catch (CloneNotSupportedException e) {
        }
        return c;
    }

    public void dump(DataOutputStream file) throws IOException {
        int size = Math.min(this.constantPool.length, 65535);
        file.writeShort(size);
        for (int i = 1; i < size; i++) {
            if (this.constantPool[i] != null) {
                this.constantPool[i].dump(file);
            }
        }
    }

    public <T extends Constant> T getConstant(int i) throws ClassFormatException {
        return (T) getConstant(i, Constant.class);
    }

    public <T extends Constant> T getConstant(int i, byte b) throws ClassFormatException {
        return (T) getConstant(i, b, Constant.class);
    }

    public <T extends Constant> T getConstant(int i, byte b, Class<T> cls) throws ClassFormatException {
        T t = (T) getConstant(i);
        if (t.getTag() != b) {
            throw new ClassFormatException("Expected class `" + Const.getConstantName(b) + "' at index " + i + " and got " + t);
        }
        return t;
    }

    public <T extends Constant> T getConstant(int index, Class<T> castTo) throws ClassFormatException {
        Constant prev;
        if (index >= this.constantPool.length || index < 0) {
            throw new ClassFormatException("Invalid constant pool reference: " + index + ". Constant pool size is: " + this.constantPool.length);
        }
        T c = castTo.cast(this.constantPool[index]);
        if (c == null && index != 0 && ((prev = this.constantPool[index - 1]) == null || (prev.getTag() != 6 && prev.getTag() != 5))) {
            throw new ClassFormatException("Constant pool at index " + index + " is null.");
        }
        return c;
    }

    public ConstantInteger getConstantInteger(int index) {
        return (ConstantInteger) getConstant(index, (byte) 3, ConstantInteger.class);
    }

    public Constant[] getConstantPool() {
        return this.constantPool;
    }

    public String getConstantString(int index, byte tag) throws IllegalArgumentException {
        int i;
        switch (tag) {
            case JavaClass.HEAP /* 1 */:
                return getConstantUtf8(index).getBytes();
            case 7:
                i = ((ConstantClass) getConstant(index, ConstantClass.class)).getNameIndex();
                break;
            case 8:
                i = ((ConstantString) getConstant(index, ConstantString.class)).getStringIndex();
                break;
            case 19:
                i = ((ConstantModule) getConstant(index, ConstantModule.class)).getNameIndex();
                break;
            case 20:
                i = ((ConstantPackage) getConstant(index, ConstantPackage.class)).getNameIndex();
                break;
            default:
                throw new IllegalArgumentException("getConstantString called with illegal tag " + ((int) tag));
        }
        return getConstantUtf8(i).getBytes();
    }

    public ConstantUtf8 getConstantUtf8(int index) throws ClassFormatException {
        return (ConstantUtf8) getConstant(index, (byte) 1, ConstantUtf8.class);
    }

    public int getLength() {
        if (this.constantPool == null) {
            return 0;
        }
        return this.constantPool.length;
    }

    @Override // java.lang.Iterable
    public Iterator<Constant> iterator() {
        return Arrays.stream(this.constantPool).iterator();
    }

    public void setConstant(int index, Constant constant) {
        this.constantPool[index] = constant;
    }

    public void setConstantPool(Constant[] constantPool) {
        this.constantPool = constantPool;
    }

    public String toString() {
        StringBuilder buf = new StringBuilder();
        for (int i = 1; i < this.constantPool.length; i++) {
            buf.append(i).append(")").append(this.constantPool[i]).append("\n");
        }
        return buf.toString();
    }
}
